"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[7947],{67947:function(o){o.exports=JSON.parse('{"base":"vs","inherit":true,"rules":[{"background":"FFFFFF","token":""},{"foreground":"919191","token":"comment"},{"foreground":"00a33f","token":"string"},{"foreground":"a535ae","token":"constant.language"},{"foreground":"ff5600","token":"keyword"},{"foreground":"ff5600","token":"storage"},{"foreground":"21439c","token":"entity.name.type"},{"foreground":"21439c","token":"entity.name.function"},{"foreground":"a535ae","token":"support.function"},{"foreground":"a535ae","token":"support.constant"},{"foreground":"a535ae","token":"support.type"},{"foreground":"a535ae","token":"support.class"},{"foreground":"a535ae","token":"support.variable"},{"foreground":"ffffff","background":"990000","token":"invalid"},{"foreground":"990000","token":"constant.other.placeholder.py"}],"colors":{"editor.foreground":"#000000","editor.background":"#FFFFFF","editor.selectionBackground":"#BAD6FD","editor.lineHighlightBackground":"#00000012","editorCursor.foreground":"#000000","editorWhitespace.foreground":"#BFBFBF"}}')}}]);