"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[6443],{76443:function(o){o.exports=JSON.parse('{"base":"vs","inherit":true,"rules":[{"background":"969696","token":""},{"foreground":"8d2e75","fontStyle":"italic","token":"comment"},{"foreground":"ffffff","background":"ff000033","token":"string"},{"foreground":"ffffff","token":"constant.numeric"},{"foreground":"ffa995","fontStyle":"bold","token":"constant.language"},{"foreground":"ffa995","background":"0000ff33","token":"constant.character"},{"foreground":"ffa995","background":"0000ff33","token":"constant.other"},{"foreground":"ffa995","token":"variable.language"},{"foreground":"ffa995","token":"variable.other"},{"fontStyle":"bold","token":"keyword"},{"foreground":"3a68a3","fontStyle":"bold","token":"storage"},{"fontStyle":"underline","token":"entity.name.type"},{"fontStyle":"italic","token":"entity.other.inherited-class"},{"foreground":"ffa995","token":"entity.name.function"},{"fontStyle":"italic","token":"variable.parameter"},{"foreground":"0000ff","fontStyle":"bold","token":"entity.name"},{"foreground":"3a68a3","fontStyle":"italic","token":"entity.other.attribute-name"},{"foreground":"e5b3ff","token":"support.function"},{"foreground":"000000","token":"support.function.any-method"},{"fontStyle":"italic","token":"support.function.any-method - punctuation"},{"foreground":"ffffff","token":"support.constant"},{"foreground":"ffa995","token":"support.type"},{"foreground":"ffa995","token":"support.class"},{"foreground":"3a68a3","token":"support.variable"},{"foreground":"ffffff","background":"797979","token":"invalid"},{"foreground":"ffa995","background":"969696","fontStyle":"italic","token":"string.quoted.other.lt-gt.include"},{"foreground":"ffa995","background":"969696","token":"string.quoted.double.include"},{"foreground":"4d4e60","token":"markup.list"},{"foreground":"ffffff","background":"0000ff","token":"markup.raw"},{"foreground":"00f0c9","token":"markup.quote"},{"foreground":"4c457e","token":"markup.quote markup.quote"},{"background":"8a9ecb","token":"text.html source"}],"colors":{"editor.foreground":"#000000","editor.background":"#969696","editor.selectionBackground":"#B1B1B1","editor.lineHighlightBackground":"#00000012","editorCursor.foreground":"#FFFFFF","editorWhitespace.foreground":"#FF38FF"}}')}}]);