"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[4776],{44776:function(o){o.exports=JSON.parse('{"base":"vs-dark","inherit":true,"rules":[{"background":"2A211C","token":""},{"foreground":"0066ff","fontStyle":"italic","token":"comment"},{"foreground":"43a8ed","fontStyle":"bold","token":"keyword"},{"foreground":"43a8ed","fontStyle":"bold","token":"storage"},{"foreground":"44aa43","token":"constant.numeric"},{"foreground":"c5656b","fontStyle":"bold","token":"constant"},{"foreground":"585cf6","fontStyle":"bold","token":"constant.language"},{"foreground":"318495","token":"variable.language"},{"foreground":"318495","token":"variable.other"},{"foreground":"049b0a","token":"string"},{"foreground":"2fe420","token":"constant.character.escape"},{"foreground":"2fe420","token":"string source"},{"foreground":"1a921c","token":"meta.preprocessor"},{"foreground":"9aff87","fontStyle":"bold","token":"keyword.control.import"},{"foreground":"ff9358","fontStyle":"bold","token":"entity.name.function"},{"foreground":"ff9358","fontStyle":"bold","token":"keyword.other.name-of-parameter.objc"},{"fontStyle":"underline","token":"entity.name.type"},{"fontStyle":"italic","token":"entity.other.inherited-class"},{"fontStyle":"italic","token":"variable.parameter"},{"foreground":"8b8e9c","token":"storage.type.method"},{"fontStyle":"italic","token":"meta.section entity.name.section"},{"fontStyle":"italic","token":"declaration.section entity.name.section"},{"foreground":"7290d9","fontStyle":"bold","token":"support.function"},{"foreground":"6d79de","fontStyle":"bold","token":"support.class"},{"foreground":"6d79de","fontStyle":"bold","token":"support.type"},{"foreground":"00af0e","fontStyle":"bold","token":"support.constant"},{"foreground":"2f5fe0","fontStyle":"bold","token":"support.variable"},{"foreground":"687687","token":"keyword.operator.js"},{"foreground":"ffffff","background":"990000","token":"invalid"},{"background":"ffd0d0","token":"invalid.deprecated.trailing-whitespace"},{"background":"f5aa7730","token":"text source"},{"background":"f5aa7730","token":"string.unquoted"},{"foreground":"8f7e65","token":"meta.tag.preprocessor.xml"},{"foreground":"888888","token":"meta.tag.sgml.doctype"},{"fontStyle":"italic","token":"string.quoted.docinfo.doctype.DTD"},{"foreground":"43a8ed","token":"meta.tag"},{"foreground":"43a8ed","token":"declaration.tag"},{"fontStyle":"bold","token":"entity.name.tag"},{"fontStyle":"italic","token":"entity.other.attribute-name"}],"colors":{"editor.foreground":"#BDAE9D","editor.background":"#2A211C","editor.selectionBackground":"#C3DCFF","editor.lineHighlightBackground":"#3A312C","editorCursor.foreground":"#889AFF","editorWhitespace.foreground":"#BFBFBF"}}')}}]);