{% extends "transformers/default.jinja" %}
{% block imports %}
from mage_ai.data_cleaner.transformer_actions.base import BaseAction
from mage_ai.data_cleaner.transformer_actions.constants import ActionType, Axis
from mage_ai.data_cleaner.transformer_actions.utils import build_transformer_action
{{ super() -}}
{% endblock %}


{% block content %}
@transformer
def execute_transformer_action(df: DataFrame, *args, **kwargs) -> DataFrame:
    {% block action %}
    action = build_transformer_action(
        df,
        action_type=ActionType.CLEAN_COLUMN_NAME,
        arguments=df.columns,
        axis=Axis.COLUMN,
    )
    {% endblock %}

    return BaseAction(action).execute(df)
{% endblock %}
