(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[327],{34376:function(n,e,i){n.exports=i(8641)},71388:function(n,e,i){"use strict";i.r(e);var r=i(77837),t=i(38860),u=i.n(t),p=i(82684),c=i(34376);function s(n){var e=n.pipeline,i=(0,c.useRouter)();(0,p.useEffect)((function(){i.replace("/pipelines/[pipeline]/triggers","/pipelines/".concat(e.uuid,"/triggers"))}),[])}s.getInitialProps=function(){var n=(0,r.Z)(u().mark((function n(e){var i;return u().wrap((function(n){for(;;)switch(n.prev=n.next){case 0:return i=e.query.pipeline,n.abrupt("return",{pipeline:{uuid:i}});case 2:case"end":return n.stop()}}),n)})));return function(e){return n.apply(this,arguments)}}(),e.default=s},67503:function(n,e,i){(window.__NEXT_P=window.__NEXT_P||[]).push(["/pipelines/[pipeline]",function(){return i(71388)}])}},function(n){n.O(0,[9774,2888,179],(function(){return e=67503,n(n.s=e);var e}));var e=n.O();_N_E=e}]);