self.__BUILD_MANIFEST=function(s,e,i,p,n,c,t,a,l,u,d,r,k,b,f,h,j,g,o,_,m,E,O,A,B){return{__rewrites:{beforeFiles:[],afterFiles:[{source:"/CLOUD_NOTEBOOK_BASE_PATH_PLACEHOLDER_/datasets",destination:b},{source:"/CLOUD_NOTEBOOK_BASE_PATH_PLACEHOLDER_",destination:b}],fallback:[]},"/":["static/chunks/pages/index-dd3f301c4936ed24.js"],"/_error":["static/chunks/pages/_error-235304e5badb19eb.js"],"/datasets":[s,i,e,m,"static/chunks/pages/datasets-2a0e69cd360c1d08.js"],"/datasets/[...slug]":[f,"static/chunks/8a28b14e-b845f39223877713.js",s,i,t,a,l,u,h,E,"static/chunks/17-6f5e28da6ea3bc27.js",e,c,d,r,j,O,m,"static/chunks/pages/datasets/[...slug]-e871cce30e39804e.js"],"/pipeline-runs":[s,e,p,n,"static/chunks/pages/pipeline-runs-2db05b1806ffa5a9.js"],"/pipelines":[s,i,e,p,n,"static/chunks/pages/pipelines-56aad4e0c3c79398.js"],"/pipelines/[pipeline]":["static/chunks/pages/pipelines/[pipeline]-0c6dc5356df3d124.js"],"/pipelines/[pipeline]/edit":[f,s,i,t,a,l,u,g,h,E,"static/chunks/2437-52e7a8f62f271f5b.js",e,p,d,k,r,j,O,"static/chunks/pages/pipelines/[pipeline]/edit-c44ca8c8134d4bda.js"],"/pipelines/[pipeline]/logs":[s,i,e,p,n,c,o,"static/chunks/pages/pipelines/[pipeline]/logs-a5f6224e176b1979.js"],"/pipelines/[pipeline]/monitors":[s,i,t,u,e,p,n,c,_,A,"static/chunks/pages/pipelines/[pipeline]/monitors-dba1111979718035.js"],"/pipelines/[pipeline]/monitors/block-runs":[s,i,t,u,e,p,n,c,r,_,A,"static/chunks/pages/pipelines/[pipeline]/monitors/block-runs-d6717c1c18ec84bd.js"],"/pipelines/[pipeline]/monitors/block-runtime":[f,s,i,t,u,"static/chunks/9832-c8b8970bb522f302.js",e,p,n,c,r,_,j,"static/chunks/pages/pipelines/[pipeline]/monitors/block-runtime-be9cac50c20d3835.js"],"/pipelines/[pipeline]/runs":[s,i,a,g,e,p,n,c,k,o,B,"static/chunks/pages/pipelines/[pipeline]/runs-e64c15fedc2f0e4f.js"],"/pipelines/[pipeline]/runs/[run]":[s,i,t,a,l,h,e,p,n,c,d,k,"static/chunks/pages/pipelines/[pipeline]/runs/[run]-bc3b7c3543cd6934.js"],"/pipelines/[pipeline]/triggers":[s,i,t,a,l,e,p,n,c,d,k,"static/chunks/pages/pipelines/[pipeline]/triggers-2cdcac93db51bf4b.js"],"/pipelines/[pipeline]/triggers/[...slug]":[s,i,t,a,l,g,"static/chunks/5119-93db164b6c40e3f7.js",e,p,n,c,d,k,r,o,B,"static/chunks/pages/pipelines/[pipeline]/triggers/[...slug]-bf806eb8f86252f7.js"],sortedPages:["/","/_app","/_error",b,"/datasets/[...slug]","/pipeline-runs","/pipelines","/pipelines/[pipeline]","/pipelines/[pipeline]/edit","/pipelines/[pipeline]/logs","/pipelines/[pipeline]/monitors","/pipelines/[pipeline]/monitors/block-runs","/pipelines/[pipeline]/monitors/block-runtime","/pipelines/[pipeline]/runs","/pipelines/[pipeline]/runs/[run]","/pipelines/[pipeline]/triggers","/pipelines/[pipeline]/triggers/[...slug]"]}}("static/chunks/5339-11f4757850474424.js","static/chunks/2874-1f0e738f0360c506.js","static/chunks/2083-31bedbe3b4490815.js","static/chunks/9056-0ab07339c8ac018b.js","static/chunks/8933-0ee8f02ff6d95031.js","static/chunks/1273-51cd6b4bca1594b7.js","static/chunks/6674-63776cbaecd7c1d9.js","static/chunks/1774-aa51ef1da7217ff9.js","static/chunks/3668-3b07d0117a5db904.js","static/chunks/2714-1e79e9f2e998b544.js","static/chunks/2151-0c54e75e20a26840.js","static/chunks/5378-08ec0e4edeb2117a.js","static/chunks/6532-f4faf76af4c79f9f.js","/datasets","static/chunks/29107295-989a0767a635d9d5.js","static/chunks/2125-d859c4773f5f76a8.js","static/chunks/8735-dbcddabbd15ab44d.js","static/chunks/7619-11107d4d093f5d51.js","static/chunks/5998-498a8d0f087d8be5.js","static/chunks/2249-ce669540b984a4ec.js","static/chunks/3887-5c8404eec6532f49.js","static/chunks/5650-7bd2f076f244e576.js","static/chunks/9223-dcbd32ce9d74ebc0.js","static/chunks/7400-bce3a3e39e0a13ed.js","static/chunks/1476-6cd1c7516868a0fd.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();