"""Add variables to pipeline schedule.

Revision ID: 52ab80005742
Revises: c07a23ff782b
Create Date: 2022-08-25 10:31:48.480241

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '52ab80005742'
down_revision = 'c07a23ff782b'
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('pipeline_schedule', sa.Column('variables', sa.JSON(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('pipeline_schedule', 'variables')
    # ### end Alembic commands ###
