(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5405],{34376:function(n,u,t){n.exports=t(8641)},63878:function(n,u,t){"use strict";t.r(u);var e=t(82684),i=t(34376);u.default=function(){var n=(0,i.useRouter)(),u=n.asPath,t=u.split("?")[0],c=u;"/"===t&&(c="/pipelines"),(0,e.useEffect)((function(){n.replace(c)}),[])}},48312:function(n,u,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/",function(){return t(63878)}])}},function(n){n.O(0,[9774,2888,179],(function(){return u=48312,n(n.s=u);var u}));var u=n.O();_N_E=u}]);